;------------------------------------------------------------------------------
; File Name :      adc41159.inc
;------------------------------------------------------------------------------
;      Copyright: Eugene M. Hutorny  2006
;      Revision:  V1.00
;      Date:      Nov 17, 2006
;      License:   Licensed to public under the Open Software License 
;                 http://www.opensource.org/licenses/osl-3.0.php
;------------------------------------------------------------------------------
;
;------------------------------------------------------------------------------
; ADC model refinement for MCU described in DS 41159
;------------------------------------------------------------------------------

 ; enum ConvClock
 constant ADC_eClockFOSC_2      = b'000'
 constant ADC_eClockFOSC_8      = b'001'
 constant ADC_eClockFOSC_32     = b'010'
 constant ADC_eClockFRC         = b'011'
 constant ADC_eClockFOSC_4      = b'100'
 constant ADC_eClockFOSC_16     = b'101'
 constant ADC_eClockFOSC_64     = b'110'

ADC_mSetClock macro ConvClock
  if ConvClock > ADC_eClockFOSC_64  + 1
        error Improper parameter passed to "ADC_mSetClock" 
  endif
  if ConvClock & b'001'
    bsf         ADCON0, ADCS0
  else  
    bcf         ADCON0, ADCS0
  endif
  if ConvClock & b'010'
    bsf         ADCON0, ADCS1
  else  
    bcf         ADCON0, ADCS1
  endif
  if ConvClock & b'100'
    bsf         ADCON1, ADCS2
  else  
    bcf         ADCON1, ADCS2
  endif
 endm

ADC_mSetVref macro VrefSrc
 if VrefSrc != ADC_eVrefVdd && VrefSrc != ADC_eVrefVref
    error Improper parameter passed to "ADC_mSetVref" 
 endif
 if VrefSrc == ADC_eVrefVdd
    bcf         ADCON1, PCFG0
 endif
 if VrefSrc == ADC_eVrefVref
    bsf         ADCON1, PCFG0
 endif
 endm

ADC_mSetChannel macro const
    movlw       ~((1 << CHS2) | (1 << CHS1) | (1 << CHS0))
    andwf       ADCON0, W
    iorlw       (const & 07h) << CHS0
    movwf       ADCON0
 endm

ADC_mSetChnlDyn macro gpr
    swapf       gpr, F          ; gpr <<= 4
    rrncf       gpr, F          ; gpr >>= 1
    movlw       ~((1 << CHS2) | (1 << CHS1) | (1 << CHS0))
    addwf       gpr, F
    andwf       ADCON0, W
    iorwf       gpr, W
    movwf       ADCON0
    rlncf       gpr, F          ; gpr <<= 1
    swapf       gpr, F          ; gpr >>= 4
 endm

ADC_mON macro
    bsf         ADCON0, ADON
 endm

ADC_mOFF macro
    bcf         ADCON0, ADON
 endm

ADC_mSetResJustf macro Justf
 if Justf != ADC_eJustfRight && Justf == ADC_eJustfLeft
    error Improper parameter passed to "ADC_mSetResJustf"
 endif
 if Justf == ADC_eJustfRight
    bsf         ADCON1, ADFM
 endif
 if Justf == ADC_eJustfLeft
    bcf         ADCON1, ADFM
 endif
 endm

ADC_mGO macro
    bsf         ADCON0, GO
 endm

ADC_mSkipDone macro
    btfsc       ADCON0, GO
 endm

ADC_mSkipNotDone macro
    btfss       ADCON0, GO
 endm

ADC_mSetAnaloginput macro const
  if const <= AN3
   if const < AN3
    bsf         TRISA, const
   else
    bsf         TRISA, RA4
   endif
  if const >= AN4 && const <= AN9
   if const <= AN7
    bsf         TRISC, const - AN4
   else
    bsf         TRISC, RC6 + const - AN8
   endif
  endif
  if const >= AN10
    bsf         TRISB, RB4 + const - AN10    
  endif
  if const <= AN7
   _sfr_banksel    ANSEL
    movlw       .1 << const
    iorwf       ANSEL
  endif
  if const >= AN8
    _sfr_banksel    ANSELH
    movlw       .1 << (const - AN8)
    iorwf       ANSELH
  endif
 endm
