;------------------------------------------------------------------------------
; File Name :     main.asm
;------------------------------------------------------------------------------
;      Copyright: Eugene M. Hutorny  2006
;      Revision:  V1.00
;      Date:      Oct 15, 2006
;      License:   Licensed to public under the Open Software License 
;                 http://www.opensource.org/licenses/osl-3.0.php
;------------------------------------------------------------------------------
; MIHA - Multi Input Hall Automaton
;------------------------------------------------------------------------------

 include "pic.inc"
 include "vos.inc"
 include "msw.inc"
 include "hla.inc"
 include "act.inc"
 include "mdl.inc"
 include "adc.inc"
 include "ppi.inc"

;------------------------------------------------------------------------------
	__CONFIG _WDT_ON & _INTRC_OSC_NOCLKOUT & _CP_OFF & _PWRTE_ON & _MCLRE_OFF
;------------------------------------------------------------------------------
    ; GP0 - potentiometer, GP1-GP4 - inputs, GP5 - output
    ; Port usage for debugging - GP4 - USART, GP2 - signal
    constant mswPORT = GPIO
    constant mswMASK = b'00011110'
    constant mswTRIS = TRISIO
    constant actPORT = GPIO
    constant actMASK = b'00100000'
    constant actTRIS = TRISIO
    constant mswBufferSize = .4 + .1
    constant ppiChannel = ANS0
    constant miha_TRISAN  = .1         ; ANS0 == RA0

;------------------------------------------------------------------------------
; Object instantiation
VOS_Objects                                             ; starts VOS object section

switchDebouncer
    VOS_mPersistentObject MSW, switchDebouncerData      ; creates an instance of MSW class 

hallAutomatonForSwitches
    VOS_mPersistentObject HLASW, hallAutomatonSWData

messageDelayLine     
    VOS_mPersistentObject MDL, messageDelayData

actuator
    VOS_mPersistentObject ACT, actuatorData

parameterInput
    VOS_mPersistentObject PPI, parameterInputData

;------------------------------------------------------------------------------
; Object data and commutation
DEEPROM code
switchDebouncerData                             ; 
    de hallAutomatonForSwitches                 ; data.accepter (logical connection)
    de mswPORT, mswMASK                         ; port and sensitivity mask 
    de switchDebouncerBuffer                    ; Working buffer

 ifdef switchDebouncerSecondary
switchDebouncerSecondaryData
    de hallAutomatonForPushButtons              ; secondary automaton works with push buttons 
    de mswPORT1, mswMASK1
    de switchDebouncerBuffer1
 endif

hallAutomatonSWData                             ; Automaton connected to:
    de actuator                                 ; to actuator
    de messageDelayLine                         ; and to the message delay line

 ifdef hallAutomatonForPushButtons
hallAutomatonPBData
    de actuator
    de messageDelayLine
 endif

messageDelayData
    de actuator

actuatorData
    de actPORT, actMASK

parameterInputData
    de messageDelayLine
    de MDL_cMsgIDSetParam
    de ppiChannel

;------------------------------------------------------------------------------
; RAM allocation
    udata
    global MSW_Buffers
MSW_Buffers
switchDebouncerBuffer res mswBufferSize    

;------------------------------------------------------------------------------
 constant miha_ANSEL   = (.1 << ppiChannel)
 constant miha_ANSELH  = .0
;------------------------------------------------------------------------------
; Application initialization
VOS_BootHandlers code                           ; starts a section for the initialization routine
miha_init
    _pic_bankinit
    _pic_banksel OSCCAL
    call        _pic_maxrom             ;Get the cal value
    movwf       OSCCAL                  ;Calibrate
    _pic_banksel ANSEL
    movlw       miha_ANSEL
    movwf       ANSEL
    _pic_banksel OPTION_REG
    bcf         OPTION_REG, NOT_GPPU
    _pic_banksel WPU
    movlw       mswMASK
    movwf       WPU
    _pic_banksel CMCON
    movlw       b'111' << CM0           ; Turn off comparator
    movwf       CMCON
 end
