;------------------------------------------------------------------------------
; File Name :      mdl.asm
;------------------------------------------------------------------------------
;      Copyright: Eugene M. Hutorny  2006
;      Revision:  V1.00
;      Date:      Oct 21, 2006
;      License:   Licensed to public under the Open Software License 
;                 http://www.opensource.org/licenses/osl-3.0.php
;------------------------------------------------------------------------------
;
;------------------------------------------------------------------------------
; Message Delay Line implementation
; 
;------------------------------------------------------------------------------
#define __mdl_asm
 include "pic.inc"
 include "vos.inc"
 include "mdl.inc"
 if VOS_cDEBUG == true
 include "dbg.inc"
 endif    
;------------------------------------------------------------------------------
VOS_Classes             ; Actuator class dispatch table
MDL VOS_mClass MDL_Handler, MDL_Init
; Design decisions
; 1. MDL accepts any direct incoming message
; 2. MDL stores the message in the internal buffer
; 3. After the time specified expires MDL sends the message to acceptor
; 4. If a new message comes earlier, previous message is lost
;------------------------------------------------------------------------------
MDL_data        udata
mdl_state       res .1          ;
#define  mdl_bReady     mdl_state, .0
#define  mdl_bTiming    mdl_state, .1
mdl_msgid       res .1          ; message storage
mdl_msgdata     res .2
mdl_accepter    res .0          ; mdl_accepter and mdl_time share one location
mdl_time        res .2          ; time for the message to be sent
mdl_period      res .2          ; timeout period

  variable mdl_cDumpPeriod = 0
;mdl_cDumpPeriod = .1
;------------------------------------------------------------------------------
MDL_code    code
;------------------------------------------------------------------------------
MDL_Handler
    switch      INDF
    case        VTMR_cMsgIDTimer1s, mdl_timer1s
    case        MDL_cMsgIDSetParam, mdl_settimeout
    ifdef VOS_cMsgIDIdle
    case        VOS_cMsgIDIdle, _s
    endif
    btfss       VOS_bStateBCast
    bra         _m
_s  return
_m
    banksel     mdl_state
    tstf        mdl_state
    skpnz
    return                          ; not ready
    ; bra       mdl_message
;------------------------------------------------------------------------------
; event==Message
;
mdl_message                         
    movff       INDF, mdl_msgid
    incf        FSR, F
    movff       INDF, mdl_msgdata
    incf        FSR, F
    movff       INDF, mdl_msgdata+.1
    movff       mdl_period, mdl_time
    incf        mdl_period+.1,W
    movwf       mdl_time+.1
    bsf         mdl_bTiming
    return

;------------------------------------------------------------------------------
; event==Set timeout
;
mdl_settimeout
    banksel     mdl_state
    bcf         mdl_bReady
    incf        FSR,F
    movff       INDF, mdl_period
    incf        FSR,F
    movff       INDF, mdl_period+.1
    tstfsz      mdl_period
    bsf         mdl_bReady
    tstfsz      mdl_period+.1
    bsf         mdl_bReady
 if (VOS_cDEBUG == true) && mdl_cDumpPeriod
    movfw       mdl_period+.1
    call        DBG_bin2hex
    call        DBG_putc
    banksel     mdl_period
    swapf       mdl_period, W
    call        DBG_bin2hex
    call        DBG_putc
    banksel     mdl_period
    movfw       mdl_period
    call        DBG_bin2hex
    call        DBG_putc
    call        DBG_bin2hex
    movlw       0Ah
    call        DBG_putc
 endif
    return

;------------------------------------------------------------------------------
; event==Time 1s
;
; assert(high(mdl_sSending) == high(mdl_sReady))
mdl_timer1s
    btfss       mdl_bTiming
    return
    ;bra         mdl_Timing
;------------------------------------------------------------------------------
mdl_Timing
    decfsz      mdl_time, F
    return
    decfsz      mdl_time+.1, F
    return
    ;bra         mdl_Sending
;------------------------------------------------------------------------------
mdl_Sending
    PDO_mReadObjData pdd_cAddrOffset
    movwf       mdl_accepter
    VOS_mPostMsgDyn mdl_accepter, mdl_msgid
    banksel     mdl_msgdata
    movff       mdl_msgdata, INDF
    incf        FSR, F
    movff       mdl_msgdata+.1, INDF
    bcf         mdl_bTiming
    return

;------------------------------------------------------------------------------
MDL_Init
    banksel     mdl_period
    clrf        mdl_period
    clrf        mdl_period+.1
    clrf        mdl_time
    clrf        mdl_time+.1
    clrf        mdl_state               ; state == not ready
    return
 end
