;------------------------------------------------------------
; File Name :      pic16.inc
;------------------------------------------------------------
;      Copyright: Eugene M. Hutorny  2003-2006
;      Revision:  V1.01
;      Date:      Aug 25, 2006
;      License:   Licensed to public under the Open Software License 
;                 http://www.opensource.org/licenses/osl-3.0.php
;------------------------------------------------------------
;
; ----------------------------------------------------------
; Family PIC16/PIC12 specific primitives
; ----------------------------------------------------------
           SUBTITLE        "Family 16 specific definitions"
; ----------------------------------------------------------
; Should be included from pic.inc only

_pic_fam                = _pic16       ; set PIC16 family
 constant _pic_step     =  .1       ;
 constant _pic_bankmask =  7Fh
 variable _pic_shrram   =  0Fh
; ----------------------------------------------------------

ei macro
    bsf  INTCON, GIE
  endm

di macro
	bcf  INTCON, GIE
  endm

movff   macro src, dst
    movfw   src
    movwf   dst
  endm

loadf macro dst, literal
    movlw   literal
    movwf   dst
  endm

loadf16 macro dst, literal
    movlw   low(literal)
    movwf   dst   
    movlw   high(literal)
    movwf   dst+1
 endm

; ----------------------------------------------------------
; INDF/FSR primitives
;
; store file into indirect location, increment FSR
INDF_put macro file			; *FSR++ = file
	movff  file, INDF
	incf   FSR, F
  endm

; load FSR with a literal pointing to a constant location
FSR_setl macro literal         ; FSR = const
    bankisel literal
	movlw  literal
	movwf  FSR
  endm  

; load FSR with a file pointing to a location
FSR_setf macro file        ; FSR = file
    movf   file, W         
    movwf  FSR
  endm

; load FSR with a file pointing to a location, set bankisel  
FSR_setfl macro file, literal       ; FSR = file
    bankisel literal
    movf   file, W         
    movwf  FSR
  endm

; ----------------------------------------------------------
PCLATH_set macro lbl
	movlw	high(lbl)
	movwf	PCLATH
 endm

; ----------------------------------------------------------
; Enhanced banksel
 variable _pic_lastbank = 0FFFFFFFFh
_pic_bankinit macro
_pic_lastbank = 0FFFFFFFFh
 endm
_pic_banksel macro SFR
    if (_pic_lastbank & ~_pic_bankmask) != (SFR & ~_pic_bankmask)
        banksel SFR
_pic_lastbank = SFR
    endif
    if SFR & ~_pic_bankmask == 0    
        errorlevel +302
    else
        errorlevel -302
    endif
 endm
 errorlevel -312

; ----------------------------------------------------------
; PIC18-PIC16 compatibility layer

_sfr_banksel macro sfr
    banksel sfr
 endm

bra macro label
    goto    label
 endm

rcall macro label
    call    label
 endm

infsnz macro file, dst
    incfsz  file, dst
    goto    $+.2
 endm

tstfsz macro file
    tstf    file
    skpz    
 endm

tstfsn macro file
    tstf    file
    skpnz    
 endm

#define code_pack code

pic_movffN macro src, dst, len
    local i
    while i < len
        movff   src+i, dst+i
i++
    endw    
 endm

; ----------------------------------------------------------
pic_mPortToTris macro ; pic_mPortToTris(port * FSR)
    bsf    FSR,.7
 endm

; ----------------------------------------------------------
; Xor based switch/case
; Usage:
; swicth file               ; 1 instruction
; case const1, label1       ; 3 instructions
; case const2, label2       ; 3 instructions

 variable switch_val
switch macro file
switch_val = 0
    movfw   file
 endm

case macro const, label
    xorlw   switch_val ^ const
    bz label
switch_val = const
 endm
